# Copyright (C) 2003 Gregoire Lejeune <gregoire.lejeune@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

module Asp
  class Server
    require 'asp/common'
    require 'xml/libxml'
    require 'xml/libxslt'
	require 'net/smtp'
    
    public
    def file
      return @oApacheRequest.filename.dup
    end

    def html_encode( data )
      return Common::html_escape( data )
    end

    def execute( filename, *args )
      code = compile(filename)
    
      tmpdir = Asp::Common::stateDir
      file = Tempfile.new(File.basename(filename) + ".", tmpdir)
      begin
        file.print(code)
        file.close
        #load(file.path, true)
        load(file.path)
      ensure                                                                                                                           
        file.close(true)                                                                                                                     
      end
    end

    def mapPath( path )
      global = @oApacheRequest.filename.dup
      local = @oApacheRequest.uri
      www = global.gsub( local, "" )
      mmm = global.gsub( /[^\/]*$/, "" )

      result = ""
      if /^\// =~ path
        result = www + path
      else
        result = File::expand_path( mmm + path )
      end

      return result
    end
    
    def mail( hMailOps, hSMTPOps=nil )
	  if hSMTPOps.nil? == true
	    hSMTPOps = Hash::new
	    hSMTPOps["server"] = ENV["MailHost"].untaint || "localhost"
	    hSMTPOps["port"]   = ENV["MailPort"].untaint || 25
      end

      body = "From: " + String( hMailOps["From"] ) + "\n" +
	         "To: " + String( hMailOps["To"] ) + "\n" +
	         "Subject: " + ( String( hMailOps["Subject"] ) || "[No Subject]" ) + "\n\n" +
			 ( String( hMailOps["Body"] ) || "" )
	  
	  begin
        smtp = Net::SMTP::new( hSMTPOps["server"], hSMTPOps["port"] )
		smtp.start( )
	    smtp.sendmail( body, hMailOps["From"], hMailOps["To"] )
		smtp.finish
	  end
    end

    def url_encode( data )
      return Common::url_encode( data )
    end

    def url( uri, hParams=nil )
      args = ""
      sep = "?"
      if hParams.nil? == false
        hParams.each_pair { |k,v|
          args += sep + self.url_encode( k ) + "=" + self.url_encode( v )
          sep = "&"
        }
      end

      return( uri + args )
    end

    def xslt( xsl_data, xml_data )
      tmpdir = Asp::Common::stateDir
      xsl_file = Tempfile.new("xsl" + ".", tmpdir)
      xml_file = Tempfile.new("xml" + ".", tmpdir)
      result = nil
      begin
        xsl_file.print( xsl_data )
        xsl_file.close
        xml_file.print( xml_data )
        xml_file.close

        xslt = XML::XSLT.file( xsl_file.path )
        xslt.doc = XML::Document.file( xml_file.path )
        out = xslt.parse
        out.apply
        result = out.to_s
      rescue => e
        result = "ERROR !" + @oApacheRequest.escape_html(e.message)
      ensure 
        xsl_file.close
        xml_file.close
      end

      return result
    end
    
    private 
    def initialize( r )
      @oApacheRequest = r
    end
  
  end
end 
